package info.kwarc.mmt.frameit.rules

import info.kwarc.mmt.api.checking._
import info.kwarc.mmt.api.{DPath, GlobalName, LocalName, ParametricRule, Rule, RuleSet}
import info.kwarc.mmt.api.frontend.Controller
import info.kwarc.mmt.api.objects._
import Conversions._
import info.kwarc.mmt.api.uom.{Simplifiability, Simplify}
import info.kwarc.mmt.lf.{Apply, ApplySpine, Arrow, OfType, Pi}
import info.kwarc.mmt.LFX.datatypes.{Listtype, Ls}
import info.kwarc.mmt.api.utils.URI

import scala.collection.mutable

// TODO: ADJUST ALL PATHS
object IfThenElseX {
  val ns: DPath = DPath(URI.http colon "mathhub.info") / "FrameIT" / "frameworld"
  val path: GlobalName = ns ? "IfThenElseX" ? "ifthenelsex"
  val rawhead: OMID = OMS(path)

  def apply(tp: Term, cond: Term, tt: Term, ft: Term): Term =
    ApplySpine(this.rawhead, tp, cond, tt, ft)
  def unapply(tm : Term) : Option[(Term, Term, Term, Term)] = tm match {
    case ApplySpine(this.rawhead, List(tp, cond, tt, ft)) => Some((tp, cond, tt, ft))
    case _ => None
  }
}

object IfThenElseRuleX extends ParametricRule {
  override def apply(controller: Controller, home: Term, args: List[Term]): Rule = args match {
    case List(truetm, falsetm) =>
      RuleSet(IfThenElseCompX(truetm, falsetm))
    case _ =>
      ???
  }

  case class IfThenElseCompX(truetm: Term, falsetm: Term) extends ComputationRule(IfThenElseX.path) {
    override def applicable(t: Term): Boolean = t match {
      case IfThenElseX(_, _, _, _) => true
      case _ => false
    }

    override def apply(check: CheckingCallback)(tm: Term, covered: Boolean)(implicit stack: Stack, history: History): Simplifiability = tm match {
      case IfThenElseX(_, cond, tt, ft) =>
        check.tryToCheckWithoutDelay(Equality(stack, check.simplify(cond), truetm, None)) match {
          case Some(true) => Simplify(tt)
          case Some(false) | None =>
            check.tryToCheckWithoutDelay(Equality(stack, check.simplify(cond), falsetm, None)) match {
              case Some(true) => Simplify(ft)
              case Some(false) | None => Simplifiability.NoRecurse
            }
        }
      case _ => Simplifiability.NoRecurse
    }
  }
}